Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;

entity ALU8_tb is
end ALU8_tb;

architecture plip of ALU8_tb is
  signal Cin, Cout : std_logic := '0';
  signal F : std_logic_vector(3 downto 0) := "0000";
  signal SRC, DST, ROP2, SUM : std_logic_vector(7 downto 0) := "00000000";

  function SLV_to_bin(s:std_logic_vector) return string is
    variable t : string(s'range);
    variable u : string(3 downto 1);
  begin
    for i in s'range loop
      u := std_logic'image(s(i));
      t(i) := u(2);
    end loop;
    return t;
  end SLV_to_bin;

begin

  tb: entity ALU8 port map (
     F0 => F(0),
     F1 => F(1),
     F2 => F(2),
     F3 => F(3),
     Cin => Cin,
     SRC => SRC,
     DST => DST,
     SUM => SUM,
     ROP2 => ROP2,
     Cout => Cout);

  bench: process
    variable j, k: integer := 0;
    variable expected : std_logic_vector(7 downto 0);
    variable t : std_logic_vector(9 downto 0);

    procedure ShowSignals is
    begin
      report " F=" & SLV_to_bin(std_logic_vector(F))
           & " SRC=" & SLV_to_bin(SRC)
           & " DST=" & SLV_to_bin(DST)
           & " Cin=" & std_logic'image(Cin)
           & " Cout=" & std_logic'image(Cout)
           & " SUM=" & SLV_to_bin(SUM)
      --   & " ROP2=" & SLV_to_bin(ROP2)
           & " expect=" & SLV_to_bin(expected)
           & " Diff=" & SLV_to_bin(t(8 downto 0) XOR (Cout & SUM));
    end ShowSignals;

  begin

    for j in 0 to 255 loop
      SRC <= std_logic_vector(to_unsigned(j, 8));
      for k in 0 to 255 loop
        DST <= std_logic_vector(to_unsigned(k, 8));

        -- Bool:

        F <= "0000";
        wait for 1 ns;
        expected := SRC  OR      DST;
        if expected /= ROP2 then ShowSignals; end if;

        F <= "0001";
        wait for 1 ns;
        expected := SRC XOR      DST;
        if expected /= ROP2 then ShowSignals; end if;

        F <= "0010";
        wait for 1 ns;
        expected := SRC AND      DST;
        if expected /= ROP2 then ShowSignals; end if;

        F <= "0011";
        wait for 1 ns;
        expected := SRC AND (NOT DST);
        if expected /= ROP2 then ShowSignals; end if;

        -- ADD:
        F <= "0111";
        Cin <= '0';
        wait for 1 ns;
        t := std_logic_vector(to_unsigned(j+k, 10));
        expected := t(7 downto 0);
        if t /= ('0' & Cout & SUM) then ShowSignals; end if;

        Cin <= '1';
        wait for 1 ns;
        t := std_logic_vector(to_unsigned(j+k+1, 10));
        expected := t(7 downto 0);
        if t /= ('0' & Cout & SUM) then ShowSignals; end if;

        -- SUB:
        F <= "0110";
        --Cin <= '1';
        wait for 1 ns;
        t := std_logic_vector(to_unsigned((j+512)-k, 10));
        expected := t(7 downto 0);
        if t(8 downto 0) /= ( not Cout & SUM) then ShowSignals; end if;

      end loop;
    end loop;

    wait;
  end process;

end plip;
