-- YGREC8/REG8_A3P.vhdl
-- created ven. nov. 10 22:02:40 CET 2017 by Yann Guidon (whygee@f-cpu.org)
-- version sam. nov. 11 00:48:04 CET 2017
-- Released under the GNU AGPLv3 license
--
-- This is the register set of the YGREC8, without the PC.
-- This version is designed for ACTEL/Microsemi A3P FPGA
-- using a modified binary tree.

Library ieee;
    use ieee.std_logic_1164.all;

entity REG8 is
  port( clk, reset, write_enable : in std_logic;
    AdrSRC, AdrDST : in std_logic_vector(2 downto 0);
    PC, D1, D2, RESULT : in std_logic_vector(7 downto 0);
    SRC, DST, A1, A2  : out std_logic_vector(7 downto 0));
end REG8;

architecture A3P of REG8 is
  Signal tA1, tA2, R1, R2, R3,
    LS1a, LS1b, LS1c, LS1d, LS2a, LS2b,
    LD1a, LD1b, LD1c, LD1d, LD2a, LD2b
      : std_logic_vector(7 downto 0);

begin

  A1 <= tA1;
  A2 <= tA2; -- temp registers because mode inout is pretty annoying

  -- Read the registers with balanced MUX trees:
  SRC  <= LS2a when AdrSRC(2)='0' else LS2b;
  LS2a <= LS1a when AdrSRC(1)='0' else LS1b;
  LS2b <= LS1c when AdrSRC(0)='0' else LS1d;
  LS1a <=   D1 when AdrSRC(0)='0' else  tA1;
  LS1b <=   D2 when AdrSRC(0)='0' else  tA2;
  LS1c <=   R1 when AdrSRC(1)='0' else   R3; --\ R3 and R2 swapped
  LS1d <=   R2 when AdrSRC(1)='0' else   PC; --/

  DST  <= LD2a when AdrDST(2)='0' else LD2b;
  LD2a <= LD1a when AdrDST(1)='0' else LD1b;
  LD2b <= LD1c when AdrDST(0)='0' else LD1d;
  LD1a <=   D1 when AdrDST(0)='0' else  tA1;
  LD1b <=   D2 when AdrDST(0)='0' else  tA2;
  LD1c <=   R1 when AdrDST(1)='0' else   R3; --\ R3 and	R2 swapped
  LD1d <=   R2 when AdrDST(1)='0' else   PC; --/

  -- RESET and WRITE:
  process (clk, reset)
  begin
    if reset='0' then
      tA1 <= "00000000";
      tA2 <= "00000000";
      R1  <= "00000000";
      R2  <= "00000000";
      R3  <= "00000000";
    else
      if rising_edge(clk) and write_enable='1' then
        case AdrDST is
          when "001" => tA1 <= RESULT; -- report "write to tA1";
          when "011" => tA2 <= RESULT; -- report "write to tA2";
          when "100" =>  R1 <= RESULT; -- report "write to  R1";
          when "101" =>  R2 <= RESULT; -- report "write to  R2";
          when "110" =>  R3 <= RESULT; -- report "write to  R3";
          when others => -- PC, D1, D2 are stored somewhere else
        end case;
      end if;
    end if;
  end process;  
end A3P;
