Library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
Library work;
    use work.all;

entity SHL8_tb is
end SHL8_tb;

architecture plip of SHL8_tb is
  signal F : std_logic_vector(3 downto 0) := "0000";
  signal SRC, DST, SHL : std_logic_vector(7 downto 0) := "00000000";

  function SLV_to_bin(s:std_logic_vector) return string is
    variable t : string(s'range);
    variable u : string(3 downto 1);
  begin
    for i in s'range loop
      u := std_logic'image(s(i));
      t(i) := u(2);
    end loop;
    return t;
  end SLV_to_bin;

begin

  tb: entity SHL8 port map (
     F0 => F(0),
     F1 => F(1),
     F2 => F(2),
     F3 => F(3),
     SRC => SRC,
     DST => DST,
     SHL => SHL);

  bench: process
    variable i, j : integer := 0;

    procedure TestVector (i : integer) is
      variable KO: integer := 0;
      variable expected, R1, R2, R3, R4 : std_logic_vector(7 downto 0);
      variable t : std_logic_vector(15 downto 0);
    begin
      F <= "1000"; wait for 1 ns; R1 := SHL;  -- SHR
         expected := std_logic_vector(unsigned(DST) srl i);
         if expected /= SHL then KO:=KO+1; end if;

      F <= "1001"; wait for 1 ns; R2 := SHL;  -- SHL
         expected := std_logic_vector(unsigned(DST) sll i);
         if expected /= SHL then KO:=KO+1; end if;

      F <= "1010"; wait for 1 ns; R3 := SHL;  -- SAR
         expected := (others => DST(7)); -- sign extension
         t := std_logic_vector(unsigned(expected & DST) srl i);
         expected := t(7 downto 0);
         if expected /= SHL then KO:=KO+1; end if;

      F <= "1011"; wait for 1 ns; R4 := SHL;  -- ROL
         expected := std_logic_vector(unsigned(DST) rol i);
         if expected /= SHL then KO:=KO+1; end if;

      if KO /= 0 then
        report    SLV_to_bin(DST)
          & " " & SLV_to_bin(SRC)
          & " " & SLV_to_bin( R1)
          & " " & SLV_to_bin( R2)
          & " " & SLV_to_bin( R3)
          & " " & SLV_to_bin( R4);
      end if;
    end TestVector;

  begin
    report "Testing SHL:";
    report "   DST      SRC      SHR      SHL      SAR       ROL";
    for i in 0 to 7 loop
      SRC <= std_logic_vector(to_unsigned(i, 8));

      for j in 0 to 255 loop
        DST <= std_logic_vector(to_unsigned(j, 8));

        TestVector(i);
      end loop;
    end loop;

    wait;
  end process;

end plip;
